from requirements_detector.detect import (  # from_setup_py,
    CouldNotParseRequirements,
    RequirementsNotFound,
    find_requirements,
    from_pyproject_toml,
    from_requirements_blob,
    from_requirements_dir,
    from_requirements_txt,
    from_setup_py,
)
